{
	open Parser
}

let digit = ['0'-'9']
let integer = digit+
let fraction = digit+
let exponent = ['e''E']['+''-']?digit+
let ident = ['a'-'z' 'A'-'Z']['a'-'z' 'A'-'Z' '0'-'9']*

(* EZMath code. Scanner will start from here *)
rule code = parse
		[' ' '\t' '\r' '\n']	{ code lexbuf }
	|   "$$"					{ text lexbuf }
	|   '%'						{ comment_in_code lexbuf}
	| 	'T'						{ TRANS }
	|   "\\pi"					{ FLOAT(3.1415926) }
	|	'e'						{ FLOAT(2.71828)}
	|   (ident as fid) [' ' '\t' '\r' '\n']*'('  { FID(fid) }
	|	'_'						{ UNDERSCORE }
	|	'('						{ LPAREN }
	|	')'						{ RPAREN }
	|	'{'						{ LBRACE }
	|	'}'						{ RBRACE }
	|	','						{ COMMA }
	|	'+'						{ PLUS }
	|	'-'						{ MINUS }
	|	'*' | "\\times"			{ TIMES }
	|	'/'						{ DIVIDE }
	|	"=="					{ EQUAL }
	|	'='						{ ASSIGN }
	|	"!="					{ NEQ }
	|	">="					{ GEQ }
	|	"<="					{ LEQ }
	|	'>'						{ GT }
	|	'<'						{ LT }
	|	"\\sum"					{ SUM }
	|	"\\prod"				{ PROD }
	|	"\\sin"					{ SIN }
	|	"\\cos"					{ COS }
	|	"\\log"					{ LOG }
	|   "\\tan"					{ TAN }
	|	"\\begin"				{ BEGIN }
	|	"\\end"					{ END }
	|   "\\cdot"				{ DOTMUL }
	|	"cases"					{ CASES }
	|	"bmatrix"				{ BMATRIX }
	|	'&'						{ AND }
	|	'^'						{ CARET }
	|	"\\\\"					{ DBACKSLASH }
	|	(integer? '.' fraction exponent | integer '.'? exponent | integer '.' fraction? | '.' fraction | integer) as num { FLOAT(float_of_string num)} (* Take integer as float number *)
	|	ident as vid { VID(vid) }
	|	_ as unknown { raise (Failure("illegal character " ^ Char.escaped unknown)) }

(* Normal Latex text *)
and text = parse
		'%'						{ comment_in_text lexbuf }
	|	"$$"					{ code lexbuf }
	|	eof 					{ EOF }
	|	_						{ text lexbuf }

(* Comment (%) in Latex text *)
and comment_in_text = parse
		'\n'					{ text lexbuf }
	|	eof 					{ EOF }
	|	_						{ comment_in_text lexbuf }

(* Comment (%) in EZMath code *)
and comment_in_code = parse
		'\n'					{ code lexbuf }
	|	eof 					{ raise (Failure("Missing end symbol $$")) }
	|	_						{ comment_in_code lexbuf }